import os

from django.conf import settings

PROJECT_PATH = str(
    getattr(settings, "PROJECT_PATH", getattr(settings, "PROJECT_DIR", "./")),
)

TRANSPILE_CACHE_PATH = os.path.join(PROJECT_PATH, ".transpile/")
TRANSPILE_TIME_PATH = os.path.join(TRANSPILE_CACHE_PATH, "time")

SETTINGS_PATHS = [str(x) for x in getattr(settings, "SETTINGS_PATHS", [])]

STATIC_ROOT = str(getattr(settings, "STATIC_ROOT", "./static/"))



import re
from urllib.parse import quote
from urllib.parse import urljoin

from django import template
from django.apps import apps
from django.contrib.staticfiles.storage import ManifestStaticFilesStorage
from django.templatetags.static import PrefixNode
from django.templatetags.static import StaticNode

from npm_mjs.tools import get_last_run

register = template.Library()


class StaticTranspileNode(StaticNode):
    @classmethod
    def handle_simple(cls, path):
        path = re.sub(r"^js/(.*)\.mjs", r"js/\1.js", path)
        if apps.is_installed("django.contrib.staticfiles"):
            from django.contrib.staticfiles.storage import staticfiles_storage

            if isinstance(staticfiles_storage, ManifestStaticFilesStorage):
                return staticfiles_storage.url(path)
            else:
                return staticfiles_storage.url(path) + "?v=%s" % get_last_run(
                    "transpile",
                )
        else:
            return urljoin(
                PrefixNode.handle_simple("STATIC_URL"),
                quote(path),
            ) + "?v=%s" % get_last_run("transpile")


@register.tag
def static(parser, token):
    """
    Join the given path with the STATIC_URL setting adding a version number
    and the location of the transpile folder.
    Usage::
        {% static path [as varname] %}
    Examples::
        {% static "js/index.mjs" %} # turns into js/index.js?v=213...
        {% static "css/style.css" %} # turns into css/style.css?v=213...
        {% static variable_with_path %}
        {% static variable_with_path as varname %}
    """
    return StaticTranspileNode.handle_token(parser, token)



import json
import os

import pyjson5
from django.apps import apps as django_apps
from django.core.management.base import BaseCommand

from npm_mjs.paths import TRANSPILE_CACHE_PATH


def deep_merge_dicts(old_dict, merge_dict, scripts=False):
    for key in merge_dict:
        if key in old_dict:
            if isinstance(old_dict[key], dict) and isinstance(merge_dict[key], dict):
                if key == "scripts":
                    deep_merge_dicts(old_dict[key], merge_dict[key], True)
                else:
                    deep_merge_dicts(old_dict[key], merge_dict[key])
            else:
                # In the scripts section, allow adding to hooks such as
                # "preinstall" and "postinstall"
                if scripts and key in old_dict:
                    old_dict[key] += " && %s" % merge_dict[key]
                else:
                    old_dict[key] = merge_dict[key]
        else:
            old_dict[key] = merge_dict[key]


class Command(BaseCommand):
    help = "Join package.json files from apps into common package.json"

    def handle(self, *args, **options):
        package = {}
        configs = django_apps.get_app_configs()
        for config in configs:
            json5_package_path = os.path.join(config.path, "package.json5")
            json_package_path = os.path.join(config.path, "package.json")
            if os.path.isfile(json5_package_path):
                with open(json5_package_path) as data_file:
                    data = pyjson5.decode(data_file.read())
            elif os.path.isfile(json_package_path):
                with open(json_package_path) as data_file:
                    data = json.loads(data_file.read())
            else:
                continue
            deep_merge_dicts(package, data)
        if not os.path.exists(TRANSPILE_CACHE_PATH):
            os.makedirs(TRANSPILE_CACHE_PATH)
        package_path = os.path.join(TRANSPILE_CACHE_PATH, "package.json")
        with open(package_path, "w") as outfile:
            json.dump(package, outfile)



import os
import re
import shutil
import tempfile

from base.management import BaseCommand
from django.core.management.commands import makemessages
from django.utils.functional import cached_property
from django.utils.jslex import JsLexer
from django.utils.jslex import Tok
from django.utils.translation import templatize

JsLexer.both_before.append(
    Tok("template", r"`([^\\]|(\\(.|\n)))*?`", next="div"),
)

# This makes makemessages create both translations for Python and JavaScript
# code in one go.
#
# Additionally, it works around xgettext not being able to work with ES2015
# template strings [1] by transpiling the JavaScript files in a special way to
# replace all template strings.
#
# [1] https://savannah.gnu.org/bugs/?50920


def prepare_js_for_gettext(js):
    """
    Convert the JavaScript source `js` into something resembling C for
    xgettext.
    What actually happens is that all the regex literals are replaced with
    "REGEX".
    """

    def escape_quotes(m):
        """Used in a regex to properly escape double quotes."""
        s = m[0]
        if s == '"':
            return r"\""
        else:
            return s

    lexer = JsLexer()
    c = []
    for name, tok in lexer.lex(js):
        if name == "regex":
            # C doesn't grok regexes, and they aren't needed for gettext,
            # so just output a string instead.
            tok = '"REGEX"'
        elif name == "string":
            # C doesn't have single-quoted strings, so make all strings
            # double-quoted.
            if tok.startswith("'"):
                guts = re.sub(r"\\.|.", escape_quotes, tok[1:-1])
                tok = '"' + guts + '"'
        elif name == "template":
            # Split the template string into chunks using the ${...} pattern
            chunks = re.findall(r"\${(.*?)}", tok)
            # Concatenate the chunks with "+"
            for chunk in chunks:
                tok = tok.replace("${" + chunk + "}", '" + ' + chunk + ' + "', 1)
            # Replace backtick-quotes with double-quotes.
            tok = '"' + tok[1:-1] + '"'
        elif name == "id":
            # C can't deal with Unicode escapes in identifiers.  We don't
            # need them for gettext anyway, so replace them with something
            # innocuous
            tok = tok.replace("\\", "U")
        c.append(tok)
    return "".join(c)


class BuildFile(makemessages.BuildFile):
    @cached_property
    def is_templatized(self):
        if self.domain == "djangojs":
            return True
        elif self.domain == "django":
            file_ext = os.path.splitext(self.translatable.file)[1]
            return file_ext != ".py"
        return False

    def preprocess(self):
        """
        Preprocess (if necessary) a translatable file before passing it to
        xgettext GNU gettext utility.
        """
        if not self.is_templatized:
            return
        with open(self.path, encoding="utf-8") as fp:
            src_data = fp.read()

        if self.domain == "djangojs":
            content = prepare_js_for_gettext(src_data)
        elif self.domain == "django":
            content = templatize(src_data, origin=self.path[2:])

        with open(self.work_path, "w", encoding="utf-8") as fp:
            fp.write(content)

    def cleanup(self):
        pass


class Command(makemessages.Command, BaseCommand):
    build_file_class = BuildFile

    def handle(self, *args, **options):
        options["ignore_patterns"] += [
            "venv",
            ".direnv",
            "node_modules",
            "static-transpile",
        ]
        options["domain"] = "django"
        super().handle(*args, **options)
        options["domain"] = "djangojs"
        self.temp_dir_out = tempfile.mkdtemp()
        self.temp_dir_in = tempfile.mkdtemp()
        super().handle(*args, **options)
        shutil.rmtree(self.temp_dir_in)
        shutil.rmtree(self.temp_dir_out)
